--[[
    Calendar_Broker by LordFarlander
    Version 3.0.26
    $Revision: 26 $

    A conversion of Calendar_Fubar by wish
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Calendar_Broker" );

CalendarBroker = LibStub( "AceAddon-3.0" ):NewAddon( "Calendar_Broker", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0" );
local qtip = LibStub( "LibQTip-1.0" );

CalendarBroker.vars = { defaultColor = { r = 1.0, g = 1.0, b = 1.0, a = 1.0, } };

function CalendarBroker:Update()
    self:OnTextUpdate();
    self:UpdateTooltip();
end--CalendarBroker:Update()

function CalendarBroker:UpdateTooltip()
    if( self.vars.qtip ) then
        self:OnTooltipUpdate();
    end--if
end--CalendarBroker:UpdateTooltip()

function CalendarBroker:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
        profile = {
            ShowDayOfWeek     = false,
            UseGEMdatabase    = true,
            DebugMessages     = false,
            DateFormat_US     = true,
            TimeFormat_12h    = true,
            MaxNumberOfDays   = 30,
            GUILDEventColor   = { a = 1.0, r = 0.0, g = 1.0, b = 0.0 },
            HOLIDAYEventColor = { a = 1.0, r = 0.0, g = 1.0, b = 1.0 },
            PLAYEREventColor  = { a = 1.0, r = 1.0, g = 1.0, b = 0.0 },
            RAIDEventColor    = { a = 1.0, r = 0.0, g = 0.0, b = 1.0 },
            SYSTEMEventColor  = { a = 1.0, r = 1.0, g = 0.0, b = 0.0 },
            ShowTimes         = false,
            ShowNumberDays    = true,
            ShowDurations     = false,
            Calendars         = self:GetCalendarDefaults();
        },
    };
    local options = {
        type = "group",
        handler = self,
        args = {
            Calendars = self:GetCalendarOptionsGroup();
            Events = {
                type = "group",
                handler = self,
                name = L["Events"],
                order = 2,
                args = {
                    ShowNumberDays = {
                        type = "toggle",
                        order = 6,
                        name = L["Show number of days until events"],
                        desc = L["Show number of days until events"],
                        width = "double",
                        get = function( info )
                                  return self.db.profile.ShowNumberDays;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.ShowNumberDays = value;
                                  info.handler:Update();
                              end,
                    },
                    ShowTimes = {
                        type = "toggle",
                        order = 7,
                        name = L["Show date and time event starts or ends"],
                        desc = L["Show date and time event starts or ends"],
                        width = "double",
                        get = function( info )
                                  return self.db.profile.ShowTimes;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.ShowTimes = value;
                                  info.handler:Update();
                              end,
                    },
                    ShowDurations = {
                        type = "toggle",
                        order = 8,
                        name = L["Show duration of events"],
                        desc = L["Show duration of events"],
                        width = "double",
                        get = function( info )
                                  return self.db.profile.ShowDurations;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.ShowDurations = value;
                                  info.handler:Update();
                              end,
                    },
                    GUILDEventColor = {
                        type = "color",
                        order = 9,
                        name = L["Guild event color"],
                        desc = L["Guild event color"],
                        hasAlpha = false,
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.GUILDEventColor.r, info.handler.db.profile.GUILDEventColor.g, info.handler.db.profile.GUILDEventColor.b, info.handler.db.profile.GUILDEventColor.a;
                              end,
                        set = function( info, r, g, b, a )
                                  info.handler.db.profile.GUILDEventColor.r = r;
                                  info.handler.db.profile.GUILDEventColor.g = g;
                                  info.handler.db.profile.GUILDEventColor.b = b;
                                  info.handler.db.profile.GUILDEventColor.a = a;
                                  info.handler:Update();
                              end,
                    },
                    HOLIDAYEventColor = {
                        type = "color",
                        order = 10,
                        name = L["Holiday event color"],
                        desc = L["Holiday event color"],
                        hasAlpha = false,
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.HOLIDAYEventColor.r, info.handler.db.profile.HOLIDAYEventColor.g, info.handler.db.profile.HOLIDAYEventColor.b, info.handler.db.profile.HOLIDAYEventColor.a;
                              end,
                        set = function( info, r, g, b, a )
                                  info.handler.db.profile.HOLIDAYEventColor.r = r;
                                  info.handler.db.profile.HOLIDAYEventColor.g = g;
                                  info.handler.db.profile.HOLIDAYEventColor.b = b;
                                  info.handler.db.profile.HOLIDAYEventColor.a = a;
                                  info.handler:Update();
                              end,
                    },
                    PLAYEREventColor = {
                        type = "color",
                        order = 11,
                        name = L["Player event color"],
                        desc = L["Player event color"],
                        hasAlpha = false,
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.PLAYEREventColor.r, info.handler.db.profile.PLAYEREventColor.g, info.handler.db.profile.PLAYEREventColor.b, info.handler.db.profile.PLAYEREventColor.a;
                              end,
                        set = function( info, r, g, b, a )
                                  info.handler.db.profile.PLAYEREventColor.r = r;
                                  info.handler.db.profile.PLAYEREventColor.g = g;
                                  info.handler.db.profile.PLAYEREventColor.b = b;
                                  info.handler.db.profile.PLAYEREventColor.a = a;
                                  info.handler:Update();
                              end,
                    },
                    RAIDEventColor = {
                        type = "color",
                        order = 12,
                        name = L["Raid event color"],
                        desc = L["Raid event color"],
                        hasAlpha = false,
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.RAIDEventColor.r, info.handler.db.profile.RAIDEventColor.g, info.handler.db.profile.RAIDEventColor.b, info.handler.db.profile.RAIDEventColor.a;
                              end,
                        set = function( info, r, g, b, a )
                                  info.handler.db.profile.RAIDEventColor.r = r;
                                  info.handler.db.profile.RAIDEventColor.g = g;
                                  info.handler.db.profile.RAIDEventColor.b = b;
                                  info.handler.db.profile.RAIDEventColor.a = a;
                                  info.handler:Update();
                              end,
                    },
                    SYSTEMEventColor = {
                        type = "color",
                        order = 13,
                        name = L["System event color"],
                        desc = L["System event color"],
                        hasAlpha = false,
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.SYSTEMEventColor.r, info.handler.db.profile.SYSTEMEventColor.g, info.handler.db.profile.SYSTEMEventColor.b, info.handler.db.profile.SYSTEMEventColor.a;
                              end,
                        set = function( info, r, g, b, a )
                                  info.handler.db.profile.SYSTEMEventColor.r = r;
                                  info.handler.db.profile.SYSTEMEventColor.g = g;
                                  info.handler.db.profile.SYSTEMEventColor.b = b;
                                  info.handler.db.profile.SYSTEMEventColor.a = a;
                                  info.handler:Update();
                              end,
                    },
                },
            },
            Options = {
                type = "group",
                handler = self,
                name = L["Options"],
                order = 1,
                args = {
                    ShowDayOfWeek = {
                        type = "toggle",
                        order = 1,
                        name = L["Show Day of the Week"],
                        desc = L["Show Day of the Week"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.ShowDayOfWeek;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.ShowDayOfWeek = value;
                                  info.handler:Update();
                              end,
                    },
                    DateFormat_US = {
                        type = "toggle",
                        order = 3,
                        name = L["Date format USA"],
                        desc = L["MM/DD/YYYY vs DD/MM/YYYY"],
                        width = "double",
                        get = function( info )
                                  return self.db.profile.DateFormat_US;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.DateFormat_US = value;
                                  info.handler:Update();
                              end,
                    },
                    TimeFormat_12h = {
                        type = "toggle",
                        order = 4,
                        name = L["Time format 12 hour"],
                        desc = L["12 hour time (AM/PM) vs 24 hour time"],
                        width = "double",
                        get = function( info )
                                  return self.db.profile.TimeFormat_12h;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.TimeFormat_12h = value;
                                  info.handler:Update();
                              end,
                    },
                    MaxNumberOfDays = {
                        type = "range",
                        order = 5,
                        name = L["Number of days shown in Upcoming Events"],
                        min = 7,
                        max = 120,
                        bigStep = 1,
                        desc = L["Number of days shown in Upcoming Events"],
                        width = "double",
                        get = function( info )
                                  return self.db.profile.MaxNumberOfDays;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.MaxNumberOfDays = value;
                                  info.handler:Update();
                              end,
                    },
                },
            },
        },
    };

    self.db = LibStub( "AceDB-3.0" ):New( "CalendarBrokerDB", defaults );

    local serverHour, serverMin = GetGameTime();
    local localTime = date( "*t" );
    local serverTime = self.SecondsFromEpoch( localTime.month, localTime.day, localTime.year, serverHour, serverMin );
    self.db.profile.ServerTimeOffset = serverTime - time();

    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
    self.db.RegisterCallback( self, "OnProfileChanged" );
    self.db.RegisterCallback( self, "OnProfileReset" );

    AceConfigReg:RegisterOptionsTable( "CalendarBroker Main Options", options );
    AceConfigReg:RegisterOptionsTable( "CalendarBroker Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "CalendarBroker Main Options", L["Calendar Broker"] );
    AceConfigDialog:AddToBlizOptions( "CalendarBroker Profile", L["Profile"], L["Calendar Broker"] );

    self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( L["Calendar Broker"], {
            type = "data source",
            icon = "Interface\\AddOns\\Calendar_Broker\\Artwork\\Calendar.blp",
            label = L["Calendar Broker"],
            text = L["Calendar Broker"], -- To get Button Bin to work correctly when autohiding text
            value = "0",
            suffix = L["events"],
            OnClick = function()
                          GameTimeFrame_OnClick();
                      end,
            OnEnter = function( frame )
                          if( frame and ( not InCombatLockdown() ) ) then
                              local tooltip;
                              local numcolumns = ( self.db.profile.ShowTimes and 1 or 0 ) + ( self.db.profile.ShowNumberDays and 1 or 0 ) + ( self.db.profile.ShowDurations and 1 or 0 );

                              if( numcolumns == 3 ) then
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 4, "LEFT", "CENTER", "CENTER", "RIGHT" );
                              elseif( numcolumns == 2 ) then
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 3, "LEFT", "CENTER", "RIGHT" );
                              elseif( numcolumns == 0 ) then
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 1, "LEFT" );
                              else
                                  tooltip = qtip:Acquire( "CalendarBroker_Tooltip", 2, "LEFT", "RIGHT" );
                              end--if
                              self.vars.qtip = tooltip;
                              self:UpdateTooltip();
                              tooltip:SmartAnchorTo( frame );
                              tooltip:Show();
                          end--if
                      end,
            OnLeave = function( frame )
                          qtip:Release( self.vars.qtip );
                          self.vars.qtip = nil;
                      end,
        } );
        
        self:OnProfileChanged();
end--CalendarBroker:OnInitialize()

function CalendarBroker:OnProfileChanged()
    self:SetupCalendarSupport();
    self:Update();
end--if

function CalendarBroker:OnProfileReset()
    self:OnProfileChanged();
end--if

function CalendarBroker:OnEnable()
    self:SetupCalendarSupport();
    self.timerHandle = self:ScheduleRepeatingTimer( self.Update, 1 * CALENDARBROKER_SECPERMIN, self );
end--CalendarBroker:OnEnable()

function CalendarBroker.TimeToText( t )
    local days   = math.floor( t / CALENDARBROKER_SECPERDAY );
    local hours  = ( t - days * CALENDARBROKER_SECPERDAY ) / CALENDARBROKER_SECPERHOUR;
    local minute = ( t - ( days * CALENDARBROKER_SECPERDAY + math.floor( hours ) * CALENDARBROKER_SECPERDAY ) ) / CALENDARBROKER_SECPERMIN;
    local textT  = "ERROR";

    hours = t / CALENDARBROKER_SECPERHOUR;
    if( t <= 0 ) then
        textT = L["NOW"];
    elseif( ( days > 1 ) and ( hours > 48 ) ) then
        textT = L["%i days"]:format( days );
    elseif( hours > 1.0 ) then
        textT = L["%.1f hours"]:format( hours );
    elseif( hours == 1 ) then
        textT = L["1 hour"];
    elseif( minute > 1 ) then
        textT = L["%.1f mins"]:format( minute );
    else
        textT = L["soon"];
    end--if
    return textT;
end--CalendarBroker.TimeToText( d, h, m )

function CalendarBroker:GetFormattedDate( seconds )
    if( self.db.profile.DateFormat_US ) then
        return date( "%m/%d/%y", seconds );
    end--if
    return date( "%d/%m/%y", seconds );
end--CalendarBroker:GetFormattedDate( seconds )

function CalendarBroker:GetFormattedTime( seconds )
    if( self.db.profile.TimeFormat_12h ) then
        return date( "%I:%M %p", seconds );
    end--if
    return date( "%H:%M", seconds );
end--CalendarBroker:GetFormattedDate( seconds )

function CalendarBroker:OnTextUpdate()
    local rText = "";
    local eventCount = 0;
    local lowestTime = nil;
    local nowTime = time();

    if( self.events ) then
        for i, v in pairs( self.events ) do
            if( ( v.startTime <= nowTime ) and ( v.endTime > nowTime ) ) then
                eventCount = eventCount + 1;
            end--if
            if( ( not lowestTime ) or ( v.endTime < lowestTime ) ) then
                rText = v.title .. " " .. self.TimeToText( v.startTime );
                lowestTime = v.endTime;
            end--if
            if( ( not lowestTime ) or ( v.startTime < lowestTime ) ) then
                rText = v.title .. " " .. self.TimeToText( v.startTime );
                lowestTime = v.startTime;
            end--if
        end--for
    end--if
    self.ldbobj.value = tostring( eventCount );
    if( eventCount == 1 ) then
        self.ldbobj.suffix = L["event"];
    else
        self.ldbobj.suffix = L["events"];
    end--if
    if( self.db.profile.ShowDayOfWeek ) then
        rText = date( "%A", nowTime ) .. " " .. rText;
    else
        rText = self:GetFormattedDate( nowTime ) .. " " .. rText;
    end--if
    self.ldbobj.text = rText;
end--CalendarBroker:OnTextUpdate()

function CalendarBroker:OnTooltipUpdate()
    local vars = self.vars;
    local tooltip = vars.qtip;
    local profile = self.db.profile;
    local ShowNumberDays = profile.ShowNumberDays;
    local ShowTimes = profile.ShowTimes;
    local ShowDurations = profile.ShowDurations;
    local numcolumns = ( ShowTimes and 1 or 0 ) + ( ShowNumberDays and 1 or 0 ) + ( ShowDurations and 1 or 0 );
    local nowTime = time();
    local MaxNumberOfDays = profile.MaxNumberOfDays * CALENDARBROKER_SECPERDAY;
       
    tooltip:Clear();
    if( numcolumns == 3 ) then
        tooltip:SetColumnLayout( 3, "LEFT", "CENTER", "CENTER", "RIGHT" );
    elseif( numcolumns == 2 ) then
        tooltip:SetColumnLayout( 3, "LEFT", "CENTER", "RIGHT" );
    elseif( numcolumns == 0 ) then
        tooltip:SetColumnLayout( 1, "LEFT" );
    else
        tooltip:SetColumnLayout( 2, "LEFT", "RIGHT" );
    end--if
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Calendar Broker"], tooltip:GetHeaderFont(), nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );

    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Current Events:"] );
    if( ShowNumberDays ) then
        line, column = tooltip:SetCell( line, column, " " );
    end--if
    if( ShowTimes ) then
        line, column = tooltip:SetCell( line, column, L["Ends:"], nil, "CENTER" );
    end--if
    if( ShowDurations ) then
        line, column = tooltip:SetCell( line, column, L["Left:"], nil, "CENTER" );
    end--if

    if( self.events ) then
        for i, v in pairs( self.events ) do
            if( ( v.startTime <= nowTime ) and ( v.endTime > nowTime ) ) then
                local color = self.db.profile[v.type .. "EventColor"] or self.vars.defaultColor;
    
                line, column = tooltip:AddLine();
                line, column = tooltip:SetCell( line, column, ("|cff%02x%02x%02x%s|r"):format( color.r * 255, color.g * 255, color.b * 255, v.title ) );
                if( ShowNumberDays ) then
                    line, column = tooltip:SetCell( line, column, " " );
                end--if
                if( ShowTimes ) then
                    line, column = tooltip:SetCell( line, column, self:GetFormattedDate( v.endTime ) .. " " .. self:GetFormattedTime( v.endTime ) );
                end--if
                if( ShowDurations ) then
                    if( v.startTime > nowTime ) then
                        line, column = tooltip:SetCell( line, column, self.TimeToText( v.length - ( v.endTime - nowTime ) ) );
                    else
                        line, column = tooltip:SetCell( line, column, self.TimeToText( v.length ) );
                    end--if
                end--if
            end--if
        end--for
    end--if
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );

    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Upcoming Events:"] );
    if( ShowNumberDays ) then
        line, column = tooltip:SetCell( line, column, L["Starts in:"] );
    end--if
    if( ShowTimes ) then
        line, column = tooltip:SetCell( line, column, L["Starts at:"], nil, "CENTER" );
    end--if
    if( ShowDurations ) then
        line, column = tooltip:SetCell( line, column, L["Length:"], nil, "CENTER" );
    end--if

    if( self.events ) then
        for i, v in pairs( self.events ) do
            if( ( v.startTime > nowTime ) and ( v.startTime <= nowTime + MaxNumberOfDays ) ) then
                local color = self.db.profile[v.type .. "EventColor"] or self.vars.defaultColor;
    
                line, column = tooltip:AddLine();
                line, column = tooltip:SetCell( line, column, ("|cff%02x%02x%02x%s|r"):format( color.r * 255, color.g * 255, color.b * 255, v.title ) );
                if( ShowNumberDays ) then
                    line, column = tooltip:SetCell( line, column, self.TimeToText( v.startTime - nowTime ) );
                end--if
                if( ShowTimes ) then
                    line, column = tooltip:SetCell( line, column, self:GetFormattedDate( v.startTime ) .. " " .. self:GetFormattedTime( v.startTime ) );
                end--if
                if( ShowDurations ) then
                    line, column = tooltip:SetCell( line, column, self.TimeToText( v.length ) );
                end--if            
            end--if
        end--for
    end--if

    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, date( "%A", nowTime ) .. "  " .. self:GetFormattedDate( nowTime ), nil, nil, tooltip:GetColumnCount() );
end--CalendarBroker:OnTooltipUpdate()


